import { Request, Response } from 'express';
import { Cart, ICart } from '../schemas/cartSchema';
import { Product } from '../schemas/productSchema';
import { Order } from '../schemas/orderSchema';
import { sendEmailasync } from '../services/sendGrid';
import { config } from 'dotenv';

config();

export async function addToCart(req: Request, res: Response) {
  try {
    const { userId, productId } = req.body;
    if (!productId || productId.length !== 24) {
      res.status(400).json({ error: 'A Valid Product id is required.' });
      return;
    }

    let [product, cart] = await getProductAndCart(productId, userId);
    console.log(`product: ${product}, cart: ${cart}`);

    if (!cart) {
      cart = await Cart.create({
        userId,
        products: {},
      });
    }
    const currentQuantity = cart.products[productId];
    if (currentQuantity === undefined) {
      cart.products[productId] = 1;
    } else {
      cart.products[productId] += 1;
    }
    cart.markModified('products');
    await cart.save();
    res.status(200).json({
      products: cart.products,
    });
  } catch (error) {
    console.error('Error adding product to cart:', error);
    res.status(500).json({ error: 'An error occurred while adding the product to the cart.' });
  }
}

async function getProductAndCart(productId: string, userId: string) {
  try {
    return Promise.all([
      Product.findById(productId),
      Cart.findOne({ userId }),
    ])
  } catch (error) {
    console.error('Error adding product to cart:', error);
    // res.status(500).json({ error: 'An error occurred while adding the product to the cart.' });
  }
}

export async function listCart(req: Request, res: Response) {
  try {
    const { userId } = req.body;
    const cart = await Cart.findOne({ userId });
    if (!cart) {
      res.status(404).json({ error: 'Cart not found.' });
      return;
    }
    res.status(200).json(cart.products);
  } catch (error) {
    console.error('Error listing cart:', error);
    res.status(500).json({ error: 'An error occurred while listing the cart.' });
  }
}

export async function clearCart(req: Request, res: Response) {
  try {
    const { userId } = req.body;
    await removeCart(userId);
    res.status(200).json({ message: 'Cart cleared successfully' });
  } catch (error) {
    console.error('Error clearing cart:', error);
    res.status(500).json({ error: 'An error occurred while clearing the cart.' });
  }
}

export async function checkout(req: Request, res: Response) {

  const { userId } = req.body;
  const usersCart = await Cart.findOne({ userId });

  if (!usersCart) {
    res.status(404).json({ error: 'Cart not found.' });
    return;
  }

  const order = await Order.create({
    userId,
    products: usersCart.products,
  });

  await removeCart(userId);
  sendEmailasync(order.id, userId);
  res.status(200).json(order);
}

async function removeCart(userId: string) {
  await Cart.deleteOne({ userId });
}

