import { Request, Response, NextFunction } from 'express';
import { createUser, loginUser, getAllUsers, deleteUser } from '../models/userModel';
import { ApiError } from '../utils/ApiError';
import jwt from 'jsonwebtoken';
import { clearJwtCookie, setJwtCookie } from '../middlewares/checkAuth';

const create = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const { email, password, address } = req.body;
    const user = await createUser({
      email,
      password,
      address
    });
    if(user instanceof ApiError) {
      return next(user);
    }
    res.status(201).json(user);
  } catch {
    const error = new ApiError('Error during user creation');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}

const login = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const { email, password } = req.body;
    const user: any = await loginUser({
      email,
      password
    });
    if(user instanceof ApiError) {
      console.log("Error in login")
      return res.status(user.statusCode).json({ error: user.message });
    }
    const payload = {
      userId: user._id
    }
    // Generate a JWT
    const token = jwt.sign(payload, process.env.JWT_SECRET as string, { expiresIn: '1d' });
    setJwtCookie(res, token);
    // Send the JWT as the response
    res.status(200).json({
      token
    });
  } catch {
    const error = new ApiError('Error during user login');
    error.statusCode = 500;
    error.status = 'fail';
    next(error)
  }
}

const logout = async (req: Request, res: Response, next: NextFunction) => {
  try {
    clearJwtCookie(res);
    res.status(200).json({ message: 'Logout successful' });
  } catch {
    const error = new ApiError('Error during user logout');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}

const getAll = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const users = await getAllUsers();
    res.status(200).json(users);
  } catch {
    const error = new ApiError('Error during user retrieval');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}

const deleteHandler = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const { id } = req.params;
    const user = await deleteUser(id);
    res.status(200).json(user);
  } catch {
    const error = new ApiError('Error during user deletion');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}


export {
  create,
  getAll,
  logout,
  login,
  deleteHandler
}